#
# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

import traceback
import sys

from gribapi import *

INPUT = '../../data/reduced_gaussian_lsm.grib1'
VERBOSE = 1  # verbose error reporting


def example():
    points = ((30, -20), (13, 234))

    f = open(INPUT)
    gid = grib_new_from_file(f)

    for lat, lon in points:
        nearest = grib_find_nearest(gid, lat, lon)[0]
        print lat, lon
        print nearest.lat, nearest.lon, nearest.value, nearest.distance, nearest.index

        four = grib_find_nearest(gid, lat, lon, is_lsm=False, npoints=4)
        for i in range(len(four)):
            print "- %d -" % i
            print four[i]

        print "-" * 100

    grib_release(gid)
    f.close()


def main():
    try:
        example()
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.

import traceback
import sys

from gribapi import *
from datetime import date

INPUT = '../../data/regular_latlon_surface_constant.grib1'
OUTPUT = 'out.set.grib'
VERBOSE = 1  # verbose error reporting


def example():
    fin = open(INPUT)
    fout = open(OUTPUT, 'w')
    gid = grib_new_from_file(fin)

    dt = date.today()
    today = "%d%02d%02d" % (dt.year, dt.month, dt.day)
    grib_set(gid, 'dataDate', int(today))
    grib_set(gid, 'centre', 80)

    centreIntVal = grib_get(gid, 'centre', int)
    centreStrVal = grib_get(gid, 'centre', str)
    dateStrVal = grib_get(gid, 'dataDate', str)
    assert(centreIntVal == 80)
    assert(centreStrVal == 'cnmc')
    assert(dateStrVal == today)
    print 'get centre as an integer - centre = %d' % centreIntVal
    print 'get centre as a string - centre = %s' % centreStrVal
    print 'get date as a string - date = %s' % dateStrVal

    # Now do the same but using set_key_vals, setting keys all at once
    grib_set_key_vals(gid, 'level=1,centre=98')  # with a String
    assert(grib_get(gid, 'centre', str) == 'ecmf')
    assert(grib_get(gid, 'level', int) == 1)

    grib_set_key_vals(gid, ['level=2', 'centre=kwbc'])  # with a Tuple
    assert(grib_get(gid, 'centre', int) == 7)
    assert(grib_get(gid, 'level', int) == 2)

    grib_set_key_vals(gid, {'level': 3, 'centre': 84})  # with a Dictionary
    assert(grib_get(gid, 'centre', str) == 'lfpw')
    assert(grib_get(gid, 'level', int) == 3)

    grib_gts_header(True)
    grib_gts_header(False)

    grib_write(gid, fout)
    grib_release(gid)
    fin.close()
    fout.close()


def main():
    try:
        example()
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



from .gribapi import *                # noqa
from .gribapi import __version__

#from .gribfile import GribFile        # noqa
#from .gribindex import GribIndex      # noqa
#from .gribmessage import GribMessage  # noqa



#
# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

import traceback
import sys

from gribapi import *

VERBOSE = 1  # verbose error reporting


def example(INPUT):
    f = open(INPUT)

    while 1:
        gid = grib_new_from_file(f)
        if gid is None:
            break

        iterid = grib_iterator_new(gid, 0)

        bitmapPresent = grib_get(gid, 'bitmapPresent')
        if bitmapPresent:
            # Get the bitmap array which contains 0s and 1s
            bitmap = grib_get_array(gid, 'bitmap', int)
            # Do some sanity checking
            assert len(bitmap) == grib_get_size(gid, 'values')
            assert len(bitmap) == grib_get(gid, 'numberOfDataPoints')

        i = 0
        while 1:
            result = grib_iterator_next(iterid)
            if not result:
                break

            [lat, lon, value] = result
            sys.stdout.write("- %d - lat=%.6e lon=%.6e value=" % (i, lat, lon))

            # Consult bitmap to see if the i'th value is missing
            if bitmapPresent and bitmap[i] == 0:
                print "missing"
            else:
                print "%.6f" % value

            i += 1

        grib_iterator_delete(iterid)
        grib_release(gid)

    f.close()


def main():
    try:
        example(sys.argv[1])
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



"""
``GribMessage`` class that implements a GRIB message that allows access to
the message's key-value pairs in a dictionary-like manner and closes the
message when it is no longer needed, coordinating this with its host file.

Author: Daniel Lee, DWD, 2014
"""

import collections

import gribapi


class IndexNotSelectedError(Exception):
    """GRIB index was requested before selecting key/value pairs."""


class GribMessage(object):
    """
    A GRIB message.

    Each ``GribMessage`` is stored as a key/value pair in a dictionary-like
    structure. It can be used in a context manager or by itself. When the
    ``GribFile`` it belongs to is closed, it closes any open ``GribMessage``s
    that belong to it. If a ``GribMessage`` is closed before its ``GribFile``
    is closed, it informs the ``GribFile`` of its closure.

    Scalar and vector values are set appropriately through the same method.

    Usage::

        >>> with GribFile(filename) as grib:
        ...     # Access shortNames of all messages
        ...     for msg in grib:
        ...         print(msg["shortName"])
        ...     # Report number of keys in message
        ...     len(msg)
        ...     # Report message size in bytes
        ...     msg.size
        ...     # Report keys in message
        ...     msg.keys()
        ...     # Check if value is missing
        ...     msg.missing("scaleFactorOfSecondFixedSurface")
        ...     # Set scalar value
        ...     msg["scaleFactorOfSecondFixedSurface"] = 5
        ...     # Check key's value
        ...     msg["scaleFactorOfSecondFixedSurface"]
        ...     # Set value to missing
        ...     msg.set_missing("scaleFactorOfSecondFixedSurface")
        ...     # Missing values raise exception when read with dict notation
        ...     msg["scaleFactorOfSecondFixedSurface"]
        ...     # Array values are set transparently
        ...     msg["values"] = [1, 2, 3]
        ...     # Messages can be written to file
        ...     with open(testfile, "w") as test:
        ...         msg.write(test)
        ...     # Messages can be cloned from other messages
        ...     msg2 = GribMessage(clone=msg)
        ...     # If desired, messages can be closed manually or used in with
        ...     msg.close()
    """
    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        """Release GRIB message handle and inform file of release."""
        # This assert should never trigger
        # assert self.gid is not None
        gribapi.grib_release(self.gid)
        if self.grib_index:
            self.grib_index.open_messages.remove(self)

    def close(self):
        """Possibility to manually close message."""
        self.__exit__(None, None, None)

    def __contains__(self, key):
        """Check whether a key is present in message."""
        return key in self.keys()

    def __len__(self):
        """Return key count."""
        return len(self.keys())

    def __getitem__(self, key):
        """Return value associated with key as its native type."""
        return self.get(key)

    def __setitem__(self, key, value):
        """
        Set value associated with key.

        If the object is iterable,
        """
        # Alternative implemented (TODO: evaluate)
        # if gribapi.grib_get_size(self.gid, key) > 1:
        #     gribapi.grib_set_array(self.gid, key, value)
        # else:
        #     gribapi.grib_set(self.gid, key, value)

        # Passed value is iterable and not string
        if (isinstance(value, collections.Iterable) and not
                isinstance(value, basestring)):
            gribapi.grib_set_array(self.gid, key, value)
        else:
            gribapi.grib_set(self.gid, key, value)

    def __iter__(self):
        return iter(self.keys())

    # Not yet implemented
    # def itervalues(self):
    #    return self.values()

    def items(self):
        """Return list of tuples of all key/value pairs."""
        return [(key, self[key]) for key in self.keys()]

    def keys(self, namespace=None):
        """Get available keys in message."""
        iterator = gribapi.grib_keys_iterator_new(self.gid, namespace=namespace)
        keys = []
        while gribapi.grib_keys_iterator_next(iterator):
            key = gribapi.grib_keys_iterator_get_name(iterator)
            keys.append(key)
        gribapi.grib_keys_iterator_delete(iterator)
        return keys

    def __init__(self, grib_file=None, clone=None, sample=None, gribindex=None):
        """
        Open a message and inform the GRIB file that it's been incremented.

        If ``grib_file`` is not supplied, the message is cloned from
        ``GribMessage`` ``clone``. If neither is supplied, the ``GribMessage``
        is cloned from ``sample``. If ``index`` is supplied as a GribIndex, the
        message is taken from the index.
        """
        #: Unique GRIB ID, for GRIB API interface
        self.gid = None
        #: File containing message
        self.grib_file = None
        #: GribIndex referencing message
        self.grib_index = None
        if grib_file is not None:
            self.gid = gribapi.grib_new_from_file(grib_file.file_handle)
            if self.gid is None:
                raise IOError("Grib file %s is exhausted" % grib_file.name)
            self.grib_file = grib_file
            self.grib_file.message += 1
            self.grib_file.open_messages.append(self)
        elif clone is not None:
            self.gid = gribapi.grib_clone(clone.gid)
        elif sample is not None:
            self.gid = gribapi.grib_new_from_samples(sample)
        elif gribindex is not None:
            self.gid = gribapi.grib_new_from_index(gribindex.iid)
            if not self.gid:
                raise IndexNotSelectedError("All keys must have selected "
                                            "values before receiving message "
                                            "from index.")
            self.grib_index = gribindex
            gribindex.open_messages.append(self)
        else:
            raise RuntimeError("Either grib_file, clone, sample or gribindex "
                               "must be provided.")

    def size(self):
        """Return size of message in bytes."""
        return gribapi.grib_get_message_size(self.gid)

    def dump(self):
        """Dump message's binary content."""
        return gribapi.grib_get_message(self.gid)

    def get(self, key, ktype=None):
        """Get value of a given key as its native or specified type."""
        if self.missing(key):
            raise KeyError("Key is missing from message.")
        if gribapi.grib_get_size(self.gid, key) > 1:
            ret = gribapi.grib_get_array(self.gid, key, ktype)
        else:
            ret = gribapi.grib_get(self.gid, key, ktype)
        return ret

    def missing(self, key):
        """Report if key is missing."""
        return bool(gribapi.grib_is_missing(self.gid, key))

    def set_missing(self, key):
        """Set a key to missing."""
        gribapi.grib_set_missing(self.gid, key)

    def write(self, outfile=None):
        """Write message to file."""
        if not outfile:
            # This is a hack because the API does not accept inheritance
            outfile = self.grib_file.file_handle
        gribapi.grib_write(self.gid, outfile)



#
# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

import traceback
import sys
import random

from gribapi import *

INPUT = '../../data/constant_field.grib1'
OUTPUT = 'out.clone.grib'
VERBOSE = 1  # verbose error reporting


def example():
    fin = open(INPUT)
    fout = open(OUTPUT, 'w')

    gid = grib_new_from_file(fin)

    nx = grib_get(gid, 'Ni')
    ny = grib_get(gid, 'Nj')

    for step in range(0, 24, 6):
        clone_id = grib_clone(gid)
        grib_set(clone_id, 'step', step)

        values = [random.random() for i in range(nx * ny)]

        grib_set_values(clone_id, values)

        grib_write(clone_id, fout)
        grib_release(clone_id)

    grib_release(gid)

    fin.close()
    fout.close()


def main():
    try:
        example()
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

from gribapi import *
import sys

input = sys.argv[1]

fid = open(input, "r")
oid = open("test", "w")

while 1:
    gid = grib_new_from_file(fid)
    if gid is None:
        break

    print grib_get(gid, "centre")
    print grib_get(gid, "centre", int)
    print grib_get(gid, "centre", float)
    print grib_get(gid, "centre", str)

    print "Nvalues = ", grib_get_size(gid, "values")

    values = grib_get_array(gid, "values")
    print values[:10]
    values = values * 2

    grib_set_double_array(gid, "values", values)

    values = grib_get_array(gid, "values")
    print values[:10]
    values = values / 2

    grib_set_double_array(gid, "values", [1])

    values = grib_get_values(gid)
    print values[:10]

    print grib_get(gid, "bitsPerValue")
    grib_set(gid, "bitsPerValue", 10)
    print grib_get(gid, "bitsPerValue")

    grib_write(gid, oid)

    grib_release(gid)

oid.close()
fid.close()



#
# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

import traceback
import sys

from gribapi import *

INPUT = '../../data/tigge/tigge_ecmf_pl_t.grib'
OUTPUT = 'out.set_missing.grib'
VERBOSE = 1  # verbose error reporting


def example():
    fin = open(INPUT)
    fout = open(OUTPUT, 'w')

    gid = grib_new_from_file(fin)

    grib_set_long(gid, "scaledValueOfFirstFixedSurface", 15)
    grib_set_long(gid, "scaleFactorOfFirstFixedSurface", 1)
    level = grib_get_double(gid, "level")
    assert(level == 1.5)

    # set type of level to surface
    grib_set(gid, 'typeOfFirstFixedSurface', 'sfc')
    grib_set_missing(gid, 'scaleFactorOfFirstFixedSurface')
    grib_set_missing(gid, 'scaledValueOfFirstFixedSurface')

    grib_write(gid, fout)

    grib_release(gid)
    fin.close()
    fout.close()


def main():
    try:
        example()
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

import sys
from gribapi import *
import random
import traceback

VERBOSE = 1
WRITE = 0


class Usage(Exception):
    def __init__(self):
        pass


def test():
    # test new from sample
    # grib_release(grib_new_from_samples("GRIB2"))

    if len(sys.argv) < 2:
        raise Usage

    infile = sys.argv[1]
    outfile = infile + ".out"
    clonefile = infile + ".clone"
    # binstrfile = infile + ".binstr"

    fid = open(infile, "r")
    while 1:
        gid = grib_new_from_file(fid)
        if not gid:
            break
        grib_release(gid)
    fid.close()

    fid = open(infile, "r")
    out = open(outfile, "w")
    # multi = open(outfile, "w")
    clone_fid = open(clonefile, "w")
    # binstr_fid = open(binstrfile, "w")

    # print "Writing from binary string to ",binstrfile
    # x = grib_read_file(fid)
    # print len(x)
    # grib_write_file(binstr_fid,x)
    # grib_close_file(binstr_fid)

    print "Operating on file '%s'" % infile

    n = grib_count_in_file(fid)
    print "Message count ", n

    # multi support test
    grib_multi_support_on()
    ismulti = "no" if (n == grib_count_in_file(fid)) else "yes"
    print "Is multi field - %s" % ismulti
    grib_multi_support_off()
    # end multi support test

    # gribex support test on/off
    print "Gribex support on/off"
    grib_gribex_mode_on()
    grib_gribex_mode_off
    # end gribex support test

    print "Browsing through messages "
    for i in range(n):
        gid = grib_new_from_file(fid)

        # grib_dump(gid)
        # grib_print(gid,"centre")

        if i == 0:
            print "Message size: ", grib_get_message_size(gid)
            nval = grib_get_size(gid, "values")
            print "Number of values in message %d is %d" % (i, nval)

            print "== %s %s %s %d ==" % (grib_get_string(gid, "shortName"),
                                         grib_get_string(gid, "name"),
                                         grib_get_string(gid, "typeOfLevel"),
                                         grib_get_long(gid, "level"))

            print "Nearest point to 10,10: "
            print grib_find_nearest(gid, 10, 10)

            print grib_find_nearest(gid, 10, 10, npoints=4)

            rand_list = []
            for i in range(0, 5):
                rand_index = random.randint(1, nval)
                rand_list.append(rand_index)
                myval = grib_get_double_element(gid, "values", rand_index)
                print "Random index value[%d] = %.8f" % (rand_index, myval)

            all4rand = grib_get_double_elements(gid, "values", rand_list)
            print "All at once index values: ", all4rand

            centre = grib_get_string(gid, "centre")
            grib_set_string(gid, "centre", "ecmf")
            new_centre = grib_get_string(gid, "centre")
            print "Before/after string centre: %s/%s" % (centre, new_centre)

            centre = grib_get_long(gid, "centre")
            grib_set_long(gid, "centre", 99)
            new_centre = grib_get_long(gid, "centre")
            print "Before/after numeric centre: %d/%d" % (centre, new_centre)

            centre = grib_get_double(gid, "centre")
            grib_set_double(gid, "centre", 9)
            new_centre = grib_get_double(gid, "centre")
            print "Before/after numeric floating point centre: %f/%f" % (centre, new_centre)

            vals = grib_get_double_array(gid, "values")
            print "Values before: ", vals[:10]
            grib_set_double_array(gid, "values", (1.0, 2.0, 3.14))
            vals = grib_get_double_array(gid, "values")
            print "Values after: ", vals[:10]

            print "Saving modified message to %s" % outfile
            if WRITE:
                grib_write(gid, out)

            print "Creating and saving a clone to %s" % clonefile
            clone_gid = grib_clone(gid)
            if WRITE:
                grib_write(clone_gid, clone_fid)
            grib_release(clone_gid)

            Ni = grib_get(gid, "Ni")
            print "Setting Ni to missing from --> ", Ni
            grib_set_missing(gid, "Ni")
            assert grib_is_missing(gid, "Ni")
            miss_Ni = grib_get(gid, "Ni")
            print "Ni is now --> ", miss_Ni
            grib_set(gid, "Ni", Ni)
            new_Ni = grib_get(gid, "Ni")
            print "Set Ni back to its original value --> ", new_Ni
            assert Ni == new_Ni

            print "Check some keys to see if they are defined"
            assert grib_is_defined(gid, "Ni")
            assert grib_is_defined(gid, "edition")
            assert not grib_is_defined(gid, "DarkThrone")

            # grib_multi_write(gid, multi)

        grib_release(gid)

    fid.close()
    out.close()
    clone_fid.close()
    print "Closed file"


def main():
    try:
        test()
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1
    except Usage:
        print "Usage: %s infile" % sys.argv[0]
        sys.exit(2)

if __name__ == "__main__":
    main()
    print "------------------------------------"



#
# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

import traceback
import sys

from gribapi import *

VERBOSE = 1  # verbose error reporting


def example():
    f = open(sys.argv[1])
    out = open(sys.argv[2], 'w')

    while 1:
        gid = grib_new_from_file(f)
        if gid is None:
            break

        message = grib_get_message(gid)

        newgid = grib_new_from_message(message)
        grib_write(newgid, out)
        grib_release(newgid)

        grib_release(gid)

    out.close()
    f.close()


def main():
    try:
        example()
    except GribInternalError, err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr, err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



#!/bin/env python
"""
Unit tests for ``PythonicGrib``.

Author: Daniel Lee, DWD, 2014
"""

import os
from tempfile import NamedTemporaryFile
import unittest

from gribfile import GribFile
from gribindex import GribIndex
from gribmessage import GribMessage
from gribmessage import IndexNotSelectedError


TESTGRIB = "../../data/high_level_api.grib2"
TEST_OUTPUT = "test-output.grib"
TEST_INDEX = "test.index"
TEST_KEYS = ("dataDate", "stepRange")
TEST_VALUES = 20110225, 0
SELECTION_DICTIONARY = {}
for i1 in range(len(TEST_KEYS)):
    SELECTION_DICTIONARY[TEST_KEYS[i1]] = TEST_VALUES[i1]
TEST_INDEX_OUTPUT = TESTGRIB
TEST_STEPRANGE = ('0', '12', '18', '24', '6')


class TestGribFile(unittest.TestCase):
    """Test GribFile functionality."""

    def test_memory_management(self):
        """Messages in GribFile can be opened and closed properly."""
        with GribFile(TESTGRIB) as grib:
            self.assertEqual(len(grib), 5)
            for i in range(len(grib)):
                msg = GribMessage(grib)
                self.assertEqual(msg["shortName"], "msl")
            self.assertEqual(len(grib.open_messages), 5)
        self.assertEqual(len(grib.open_messages), 0)

    def test_iteration_works(self):
        """The GribFile allows proper iteration over all messages."""
        step_ranges = []
        with GribFile(TESTGRIB) as grib:
            for _ in range(len(grib)):
                msg = GribMessage(grib)
                step_ranges.append(msg["stepRange"])
        self.assertSequenceEqual(step_ranges, ["0", "6", "12", "18", "24"])

    def test_iterator_protocol(self):
        """The GribFile allows pythonic iteration over all messages."""
        step_ranges = []
        with GribFile(TESTGRIB) as grib:
            for msg in grib:
                step_ranges.append(msg["stepRange"])
        self.assertSequenceEqual(step_ranges, ["0", "6", "12", "18", "24"])

    def test_read_past_last_message(self):
        """Trying to open message on exhausted GRIB file raises IOError."""
        with GribFile(TESTGRIB) as grib:
            for _ in range(len(grib)):
                GribMessage(grib)
            self.assertRaises(IOError, lambda: GribMessage(grib))

    def test_read_invalid_file(self):
        """Trying to open message on nonexistent GRIB file raises IOError."""
        with NamedTemporaryFile(mode='r') as f:
            with GribFile(f.name) as grib:
                self.assertRaises(IOError, lambda: GribMessage(grib))


class TestGribMessage(unittest.TestCase):
    """Test GribMessage functionality."""

    def test_metadata(self):
        """Metadata is read correctly from GribMessage."""
        with GribFile(TESTGRIB) as grib:
            msg = GribMessage(grib)
            key_count = 255
            self.assertEqual(len(msg), key_count)
            self.assertEqual(msg.size(), 160219)
            self.assertEqual(len(msg.keys()), key_count)

    def test_missing_message_behavior(self):
        """Missing messages are detected properly."""
        with GribFile(TESTGRIB) as grib:
            msg = GribMessage(grib)
            self.assertTrue(msg.missing("scaleFactorOfSecondFixedSurface"))
            msg["scaleFactorOfSecondFixedSurface"] = 5
            msg.set_missing("scaleFactorOfSecondFixedSurface")
            with self.assertRaises(KeyError):
                msg["scaleFactorOfSecondFixedSurface"]

    def test_value_setting(self):
        """Keys can be set properly."""
        with GribFile(TESTGRIB) as grib:
            msg = GribMessage(grib)
            msg["scaleFactorOfSecondFixedSurface"] = 5
            msg["values"] = [1, 2, 3]

    def test_serialize(self):
        """Message can be serialized to file."""
        with GribFile(TESTGRIB) as grib:
            msg = GribMessage(grib)
            with open(TEST_OUTPUT, "w") as test:
                msg.write(test)
        os.unlink(TEST_OUTPUT)

    def test_clone(self):
        """Messages can be used to produce clone Messages."""
        with GribFile(TESTGRIB) as grib:
            msg = GribMessage(grib)
            msg2 = GribMessage(clone=msg)
            self.assertSequenceEqual(msg.keys(), msg2.keys())


class TestGribIndex(unittest.TestCase):
    """Test GribIndex functionality."""

    def test_memory_management(self):
        """GribIndex closes GribMessages properly."""
        with GribIndex(TESTGRIB, TEST_KEYS) as idx:
            idx.select(SELECTION_DICTIONARY)
            self.assertEqual(len(idx.open_messages), 1)
        self.assertEqual(len(idx.open_messages), 0)

    def test_create_and_serialize_index(self):
        """GribIndex can be saved to file, file can be added to index."""
        with GribIndex(TESTGRIB, TEST_KEYS) as idx:
            idx.write(TEST_INDEX)
        with GribIndex(file_index=TEST_INDEX) as idx:
            idx.add(TESTGRIB)
        os.unlink(TEST_INDEX)

    # TODO: Following test disabled due to error message:
    #   GRIB_API ERROR: please select a value for index key "dataDate"
    # Must investigate
    #
    def _test_index_comprehension(self):
        """GribIndex understands underlying GRIB index properly."""
        with GribIndex(TESTGRIB, TEST_KEYS) as idx:
            self.assertEqual(idx.size(TEST_KEYS[1]), 5)
            self.assertSequenceEqual(idx.values(TEST_KEYS[1]), TEST_STEPRANGE)
            with self.assertRaises(IndexNotSelectedError):
                idx.select({TEST_KEYS[1]: TEST_VALUES[0]})
            idx.select(SELECTION_DICTIONARY)
            self.assertEqual(len(idx.open_messages), 1)

if __name__ == "__main__":
    unittest.main()



"""
``GribFile`` class that implements a GRIB file that closes itself and its
messages when it is no longer needed.

Author: Daniel Lee, DWD, 2014
"""

import gribapi
from gribmessage import GribMessage


class GribFile(file):
    """
    A GRIB file handle meant for use in a context manager.

    Individual messages can be accessed using the ``next`` method. Of course,
    it is also possible to iterate over each message in the file::

        >>> with GribFile(filename) as grib:
        ...     # Print number of messages in file
        ...     len(grib)
        ...     # Open all messages in file
        ...     for msg in grib:
        ...         print(msg["shortName"])
        ...     len(grib.open_messages)
        >>> # When the file is closed, any open messages are closed
        >>> len(grib.open_messages)
    """

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        """Close all open messages, release GRIB file handle and close file."""
        while self.open_messages:
            self.open_messages.pop().close()
        self.file_handle.close()

    def close(self):
        """Possibility to manually close file."""
        self.__exit__(None, None, None)

    def __len__(self):
        """Return total messages in GRIB file."""
        return gribapi.grib_count_in_file(self.file_handle)

    def __iter__(self):
        return self

    def next(self):
        try:
            return GribMessage(self)
        except IOError:
            raise StopIteration()

    def __init__(self, filename, mode="r"):
        """Open file and receive GRIB file handle."""
        #: File handle for working with actual file on disc
        #: The class holds the file it works with because the GRIB API's
        #: typechecking does not allow using inherited classes.
        self.file_handle = open(filename, mode)
        #: Number of message in GRIB file currently being read
        self.message = 0
        #: Open messages
        self.open_messages = []
